@@ -3,4 +3,4 @@ try:
     from django.db.models.constants import LOOKUP_SEP
 except ImportError:
     # Django <= 1.4
-    from django.db.models.sql.constants import LOOKUP_SEP
+    from django.db.models.sql.constants import LOOKUP_SEP  # noqa
